@@ -217,10 +217,14 @@
 }
 
 # return updated object with option list AND dot list
-.update.options = function(object, options, ...) {
+# optionally may exclude any opts in 'exclude'
+.update.options = function(object, options, ..., exclude) {
     if (!is.list(options))
         options = as.list(options)
     dot.opts = .match.dots.list(.valid.misc, ...)
+    if (!missing(exclude))
+        for (nm in exclude)
+            dot.opts[[nm]] = NULL
     # entries in both lists are overridden by those in ...
     for (nm in names(dot.opts))
         options[[nm]] = dot.opts[[nm]]
@@ -354,7 +358,7 @@ model.frame = function(formula, data, ...) {
 # Ideally, X is already a qr object in which case weights is assumed already incorporated
 # assign should be right if input is generated by model.matrix() or is $qr slot of lm
 #' @export
-.cmpMM = function(X, weights = diag(nrow(X)), assign = attr(X$qr, "assign")) {
+.cmpMM = function(X, weights = rep(1, nrow(X)), assign = attr(X$qr, "assign")) {
     if (!get_emm_option("enable.submodel"))
         return(NULL)
     if(!is.qr(X)) {
